--settings
local difficulty = 2 -- 1 to 3
local complexity_min = 2
local complexity_max = 4
local number_min = 5
local number_max = 20

--[[
	diffiulties:
		1 = + and -
		2 = * and /
		3 = ^

	complexity is the number of operators in the equation

	max numbers are inclusive
]]

local can_interact = false
local eq = nil

local function _gen_extra_answers(count)
    local result = {}
    local target = tonumber(eq.result)
	local modifier = complexity_max * math.ceil(math.sqrt(number_max - number_min))
    while count > 0 do
        local num = ((math.random(1,100)/100) * modifier)
        if math.random() > 0.5 then
            num = num * -1
        end
        if num - math.floor(num) > 0.5 then
            num = math.ceil(num)
        else
            num = math.floor(num)
        end
        if num ~= target then
            table.insert(result,(target + num))
            count = count - 1
        end
    end
    return result
end

local function _calculate(num1, num2, operator)
	if operator == "+" then
		return num1 + num2
	end
	if operator == "-" then
		return num1 - num2
	end
	if operator == "*" then
		return num1 * num2
	end
	if operator == "/" then
		return num1 / num2
	end
	if operator == "^" then
		return num1 ^ num2
	end
end

local function _generate_equation()
	local result = {
		equation = "",
		result = "",
	}
	local operators = {
		{ "+", "-" },
		{ "/", "*" },
		{ "^" }
	}
	local equation = {}

	-- generating operators and numbers
	table.insert(equation, math.random(number_min, number_max))
	local _cmp = math.random(complexity_min, complexity_max)
	for i = 1, _cmp, 1 do
		local op_tier = operators[math.random(difficulty)]
		table.insert(equation, op_tier[math.random(#op_tier)])
		table.insert(equation, math.random(number_min, number_max))
	end
	for i, item in ipairs(equation) do
		result.equation = result.equation .. tostring(item) .. " "
	end

	-- high priority operations
	local hpc = 0
	for i = 2, #equation, 2 do
		local item = equation[i]
		if item == "/" or item == "*" or item == "^" then
			hpc = hpc + 1
		end
	end
	for i = 1, hpc, 1 do
		local next = 2
		for j = 2, #equation, 2 do
			local item = equation[j]
			if item == "/" or item == "*" or item == "^" then
				next = j
				break
			end
		end
		local _res = _calculate(equation[next - 1], equation[next + 1], equation[next])
		local new_equation = {}
		for j = 1, next - 2, 1 do
			table.insert(new_equation, equation[j])
		end
		table.insert(new_equation, _res)
		for j = next + 2, #equation, 1 do
			table.insert(new_equation, equation[j])
		end
		equation = new_equation
	end

	-- low priority operations
	while #equation > 1 do
		local _res = _calculate(equation[1], equation[3], equation[2])
		local new_equation = {}
		table.insert(new_equation, _res)
		for j = 4, #equation, 1 do
			table.insert(new_equation, equation[j])
		end
		equation = new_equation
	end
	local _result = equation[1]
	if _result - math.floor(_result) > 0.5 then
		_result = math.ceil(_result)
	else
		_result = math.floor(_result)
	end
	result.result = tostring(_result)
	return result
end

local function _correct_callback()
	can_interact = true
	eq = nil
end

local function _incorrect_callback()
	eq = _generate_equation()
end

local function show_gui(answer,extras)
	local options = {}
	local c_pos = math.random(1,#extras+1)
	for i, item in ipairs(extras) do
		table.insert(options,{
			text = item,
			callback = _incorrect_callback
		})
	end
	table.insert(options,c_pos,{text = answer,callback = _correct_callback})
	table.insert(options,{
		text = "back",
		is_cancel_button = true
	})
	QuickMenu:new("Math menu",eq.equation,options,true)
end

local function _interact()
	if can_interact then
		can_interact = false
		return false
	end
	if eq == nil then
		eq = _generate_equation()
	end
	show_gui(eq.result,_gen_extra_answers(math.random(complexity_min,complexity_max)))
	return true
end

-- interaction override
function ObjectInteractionManager:interact(player, data, hand_id)
	if alive(self._active_unit) then
		if _interact() then
			return false
		end
		if _G.IS_VR and self._interact_hand ~= hand_id then
			return false
		end
		local interacted, timer = self._active_unit:interaction():interact_start(player, data)
		if timer then
			self._active_object_locked_data = true
		end
		return interacted or interacted == nil or false, timer, self._active_unit
	end

	return false
end
